require 'cloud_former/version'
require 'cloud_former/has_properties_and_attributes'
require 'cloud_former/makes_json'

module CloudFormer
  def self.types
    @types ||= {}
  end

  def self.type_to_class(type)
    mod, name = types[type]
    return nil if mod.nil?
    value = mod.const_get(name)
    value.is_a?(Class) && value < Resource ? value : nil
  end

  refine Module do
    def autoload(constant_name, path)
      super
      namespace = self.name.to_s.sub(/\ACloudFormer::/, '').gsub(/([a-z])([A-Z])/, '\\1_\\2').gsub('::', '.').downcase
      base_name = constant_name.to_s.gsub(/([a-z])([A-Z])/, '\\1_\\2').downcase
      ::CloudFormer.types["#{namespace}.#{base_name}"] = [self, constant_name]
    end
  end

  using self

  autoload :Boolean, 'cloud_former/boolean'
  autoload :Parameter, 'cloud_former/parameters/parameter'
  autoload :CommaDelimitedListParameter, 'cloud_former/parameters/comma_delimited_list_parameter'
  autoload :NumberParameter, 'cloud_former/parameters/number_parameter'
  autoload :StringParameter, 'cloud_former/parameters/string_parameter'

  autoload :PseudoParameter, 'cloud_former/pseudo_parameters/pseudo_parameter'
  autoload :AccountIdParameter, 'cloud_former/pseudo_parameters/account_id_parameter'
  autoload :NotificationARNsParameter, 'cloud_former/pseudo_parameters/notification_arns_parameter'
  autoload :NoValueParameter, 'cloud_former/pseudo_parameters/no_value_parameter'
  autoload :RegionParameter, 'cloud_former/pseudo_parameters/region_parameter'
  autoload :StackIdParameter, 'cloud_former/pseudo_parameters/stack_id_parameter'
  autoload :StackNameParameter, 'cloud_former/pseudo_parameters/stack_name_parameter'

  autoload :Function, 'cloud_former/functions/function'
  autoload :MetadataResource, 'cloud_former/metadata_resources/metadata_resource'
  autoload :Resource, 'cloud_former/resources/resource'
  autoload :ResourceProperty, 'cloud_former/resource_properties/resource_property'
  autoload :List, 'cloud_former/resources/list'

  module AutoScaling
    autoload :AutoScalingGroup, 'cloud_former/resources/auto_scaling/auto_scaling_group'
    autoload :BlockDeviceMapping, 'cloud_former/resource_properties/auto_scaling/block_device_mapping'
    autoload :EbsBlockDevice, 'cloud_former/resource_properties/auto_scaling/ebs_block_device'
    autoload :LaunchConfiguration, 'cloud_former/resources/auto_scaling/launch_configuration'
    autoload :LifecycleHook, 'cloud_former/resources/auto_scaling/lifecycle_hook'
    autoload :MetricsCollection, 'cloud_former/resource_properties/auto_scaling/metrics_collection'
    autoload :NotificationConfiguration, 'cloud_former/resource_properties/auto_scaling/notification_configuration'
    autoload :ScalingPolicy, 'cloud_former/resources/auto_scaling/scaling_policy'
    autoload :ScheduledAction, 'cloud_former/resources/auto_scaling/scheduled_action'
    autoload :Tag, 'cloud_former/resource_properties/auto_scaling/tag'
    autoload :UpdatePolicy, 'cloud_former/resource_properties/auto_scaling/update_policy'
  end

  module CloudFormation
    autoload :Authentication, 'cloud_former/metadata_resources/cloud_formation/authentication'
    autoload :AuthenticationItem, 'cloud_former/metadata_resources/cloud_formation/authentication_item'
    autoload :Init, 'cloud_former/metadata_resources/cloud_formation/init'
    autoload :ResourceTag, 'cloud_former/resource_properties/cloud_formation/resource_tag'
    autoload :Stack, 'cloud_former/resources/cloud_formation/stack'
    autoload :WaitConditionHandle, 'cloud_former/resources/cloud_formation/wait_condition_handle'
    autoload :WaitCondition, 'cloud_former/resources/cloud_formation/wait_condition'

    autoload :ConfigSet, 'cloud_former/metadata_resources/cloud_formation/config_set'
    autoload :ConfigSetItem, 'cloud_former/metadata_resources/cloud_formation/config_set_item'
    autoload :ConfigSetCommand, 'cloud_former/metadata_resources/cloud_formation/config_set_command'
    autoload :ConfigSetFile, 'cloud_former/metadata_resources/cloud_formation/config_set_file'
    autoload :ConfigSetGroup, 'cloud_former/metadata_resources/cloud_formation/config_set_group'
    autoload :ConfigSetPackage, 'cloud_former/metadata_resources/cloud_formation/config_set_package'
    autoload :ConfigSetService, 'cloud_former/metadata_resources/cloud_formation/config_set_service'
    autoload :ConfigSetSource, 'cloud_former/metadata_resources/cloud_formation/config_set_source'
    autoload :ConfigSetUser, 'cloud_former/metadata_resources/cloud_formation/config_set_user'
  end

  module CloudFront
    autoload :CacheBehavior, 'cloud_former/resource_properties/cloud_front/cache_behavior'
    autoload :CustomErrorResponse, 'cloud_former/resource_properties/cloud_front/custom_error_response'
    autoload :CustomOrigin, 'cloud_former/resource_properties/cloud_front/custom_origin'
    autoload :DefaultCacheBehavior, 'cloud_former/resource_properties/cloud_front/default_cache_behavior'
    autoload :Distribution, 'cloud_former/resources/cloud_front/distribution'
    autoload :DistributionConfiguration, 'cloud_former/resource_properties/cloud_front/distribution_configuration'
    autoload :ForwardedValues, 'cloud_former/resource_properties/cloud_front/forwarded_values'
    autoload :ForwardedValuesCookies, 'cloud_former/resource_properties/cloud_front/forwarded_values_cookies'
    autoload :GeoRestriction, 'cloud_former/resource_properties/cloud_front/geo_restriction'
    autoload :Logging, 'cloud_former/resource_properties/cloud_front/logging'
    autoload :Origin, 'cloud_former/resource_properties/cloud_front/origin'
    autoload :Restriction, 'cloud_former/resource_properties/cloud_front/restriction'
    autoload :S3Origin, 'cloud_former/resource_properties/cloud_front/s3_origin'
    autoload :ViewerCertificate, 'cloud_former/resource_properties/cloud_front/viewer_certificate'
  end

  module CloudTrail
    autoload :Trail, 'cloud_former/resources/cloud_trail/trail'
  end

  module CloudWatch
    autoload :Alarm, 'cloud_former/resources/cloud_watch/alarm'
    autoload :MetricDimension, 'cloud_former/resource_properties/cloud_watch/metric_dimension'
  end

  autoload :Condition, 'cloud_former/conditions/condition'

  module DirectoryService
    autoload :MicrosoftAD, 'cloud_former/resources/directory_service/microsoft_ad'
    autoload :SimpleAD, 'cloud_former/resources/directory_service/simple_ad'
    autoload :VpcSettings, 'cloud_former/resource_properties/directory_service/vpc_settings'
  end

  module EC2
    autoload :BlockDeviceMapping, 'cloud_former/resource_properties/ec2/block_device_mapping'
    autoload :CustomerGateway, 'cloud_former/resources/ec2/customer_gateway'
    autoload :DHCPOptions, 'cloud_former/resources/ec2/dhcp_options'
    autoload :EBSBlockDevice, 'cloud_former/resource_properties/ec2/ebs_block_device'
    autoload :EIP, 'cloud_former/resources/ec2/eip'
    autoload :EIPAssociation, 'cloud_former/resources/ec2/eip_association'
    autoload :ICMP, 'cloud_former/resource_properties/ec2/icmp'
    autoload :Instance, 'cloud_former/resources/ec2/instance'
    autoload :InternetGateway, 'cloud_former/resources/ec2/internet_gateway'
    autoload :MountPoint, 'cloud_former/resource_properties/ec2/mount_point'
    autoload :NetworkAcl, 'cloud_former/resources/ec2/network_acl'
    autoload :NetworkAclEntry, 'cloud_former/resources/ec2/network_acl_entry'
    autoload :NetworkInterface, 'cloud_former/resources/ec2/network_interface'
    autoload :NetworkInterfaceAttachment, 'cloud_former/resources/ec2/network_interface_attachment'
    autoload :NetworkInterfaceEmbedded, 'cloud_former/resource_properties/ec2/network_interface_embedded'
    autoload :PortRange, 'cloud_former/resource_properties/ec2/port_range'
    autoload :PrivateIpAddressSpecification, 'cloud_former/resource_properties/ec2/private_ip_address_specification'
    autoload :Route, 'cloud_former/resources/ec2/route'
    autoload :RouteTable, 'cloud_former/resources/ec2/route_table'
    autoload :SecurityGroup, 'cloud_former/resources/ec2/security_group'
    autoload :SecurityGroupEgress, 'cloud_former/resources/ec2/security_group_egress'
    autoload :SecurityGroupIngress, 'cloud_former/resources/ec2/security_group_ingress'
    autoload :SecurityGroupRule, 'cloud_former/resource_properties/ec2/security_group_rule'
    autoload :Subnet, 'cloud_former/resources/ec2/subnet'
    autoload :SubnetNetworkAclAssociation, 'cloud_former/resources/ec2/subnet_network_acl_association'
    autoload :SubnetRouteTableAssociation, 'cloud_former/resources/ec2/subnet_route_table_association'
    autoload :Tag, 'cloud_former/resource_properties/ec2/tag'
    autoload :Volume, 'cloud_former/resources/ec2/volume'
    autoload :VolumeAttachment, 'cloud_former/resources/ec2/volume_attachment'
    autoload :VPC, 'cloud_former/resources/ec2/vpc'
    autoload :VPCDHCPOptionsAssociation, 'cloud_former/resources/ec2/vpc_dhcp_options_association'
    autoload :VPCEndpoint, 'cloud_former/resources/ec2/vpc_endpoint'
    autoload :VPCGatewayAttachment, 'cloud_former/resources/ec2/vpc_gateway_attachment'
    autoload :VPCPeeringConnection, 'cloud_former/resources/ec2/vpc_peering_connection'
    autoload :VPNConnection, 'cloud_former/resources/ec2/vpn_connection'
    autoload :VPNConnectionRoute, 'cloud_former/resources/ec2/vpn_connection_route'
    autoload :VPNGateway, 'cloud_former/resources/ec2/vpn_gateway'
    autoload :VPNGatewayRoutePropagation, 'cloud_former/resources/ec2/vpn_gateway_route_propagation'
  end

  module EFS
    autoload :FileSystem, 'cloud_former/resources/efs/file_system'
    autoload :FileSystemTag, 'cloud_former/resource_properties/efs/file_system_tag'
    autoload :MountTarget, 'cloud_former/resources/efs/mount_target'
  end

  module ElastiCache
    autoload :CacheCluster, 'cloud_former/resources/elasti_cache/cache_cluster'
    autoload :ParameterGroup, 'cloud_former/resources/elasti_cache/parameter_group'
    autoload :SecurityGroup, 'cloud_former/resources/elasti_cache/security_group'
    autoload :SecurityGroupIngress, 'cloud_former/resources/elasti_cache/security_group_ingress'
    autoload :SubnetGroup, 'cloud_former/resources/elasti_cache/subnet_group'
  end

  module ElasticBeanstalk
    autoload :Application, 'cloud_former/resources/elastic_beanstalk/application'
    autoload :ApplicationVersion, 'cloud_former/resources/elastic_beanstalk/application_version'
    autoload :ConfigurationTemplate, 'cloud_former/resources/elastic_beanstalk/configuration_template'
    autoload :Environment, 'cloud_former/resources/elastic_beanstalk/environment'
    autoload :EnvironmentTier, 'cloud_former/resource_properties/elastic_beanstalk/environment_tier'
    autoload :OptionSetting, 'cloud_former/resource_properties/elastic_beanstalk/option_setting'
    autoload :SourceBundle, 'cloud_former/resource_properties/elastic_beanstalk/source_bundle'
    autoload :SourceConfiguration, 'cloud_former/resource_properties/elastic_beanstalk/source_configuration'
  end

  module ElasticLoadBalancing
    autoload :AccessLoggingPolicy, 'cloud_former/resource_properties/elastic_load_balancing/access_logging_policy'
    autoload :AppCookieStickinessPolicy, 'cloud_former/resource_properties/elastic_load_balancing/app_cookie_stickiness_policy'
    autoload :ConnectionDrainingPolicy, 'cloud_former/resource_properties/elastic_load_balancing/connection_draining_policy'
    autoload :ConnectionSettings, 'cloud_former/resource_properties/elastic_load_balancing/connection_settings'
    autoload :HealthCheck, 'cloud_former/resource_properties/elastic_load_balancing/health_check'
    autoload :LbCookieStickinessPolicy, 'cloud_former/resource_properties/elastic_load_balancing/lb_cookie_stickiness_policy'
    autoload :Listener, 'cloud_former/resource_properties/elastic_load_balancing/listener'
    autoload :LoadBalancer, 'cloud_former/resources/elastic_load_balancing/load_balancer'
    autoload :Policy, 'cloud_former/resource_properties/elastic_load_balancing/policy'
  end

  module Functions
    autoload :And, 'cloud_former/functions/and'
    autoload :Base64, 'cloud_former/functions/base_64'
    autoload :Equals, 'cloud_former/functions/equals'
    autoload :FindInMap, 'cloud_former/functions/find_in_map'
    autoload :GetAtt, 'cloud_former/functions/get_att'
    autoload :GetAZs, 'cloud_former/functions/get_azs'
    autoload :If, 'cloud_former/functions/if'
    autoload :Join, 'cloud_former/functions/join'
    autoload :Not, 'cloud_former/functions/not'
    autoload :Or, 'cloud_former/functions/or'
  end

  module IAM
    autoload :AccessKey, 'cloud_former/resources/iam/access_key'
    autoload :Group, 'cloud_former/resources/iam/group'
    autoload :IAMPolicy, 'cloud_former/resource_properties/iam/iam_policy'
    autoload :InstanceProfile, 'cloud_former/resources/iam/instance_profile'
    autoload :Policy, 'cloud_former/resources/iam/policy'
    autoload :Role, 'cloud_former/resources/iam/role'
    autoload :User, 'cloud_former/resources/iam/user'
    autoload :UserToGroupAddition, 'cloud_former/resources/iam/user_to_group_addition'
  end

  module Kinesis
    autoload :Stream, 'cloud_former/resources/kinesis/stream'
  end

  module Lambda
    autoload :Code, 'cloud_former/resource_properties/lambda/code'
    autoload :EventSourceMapping, 'cloud_former/resources/lambda/event_source_mapping'
    autoload :Function, 'cloud_former/resources/lambda/function'
    autoload :Permission, 'cloud_former/resources/lambda/permission'
  end

  module RDS
    autoload :DBInstance, 'cloud_former/resources/rds/db_instance'
    autoload :DBParameterGroup, 'cloud_former/resources/rds/db_parameter_group'
    autoload :DBSecurityGroup, 'cloud_former/resources/rds/db_security_group'
    autoload :DBSubnetGroup, 'cloud_former/resources/rds/db_subnet_group'
    autoload :SecurityGroupRule, 'cloud_former/resource_properties/rds/security_group_rule'
  end

  module Redshift
    autoload :Cluster, 'cloud_former/resources/redshift/cluster'
    autoload :ClusterParameterGroup, 'cloud_former/resources/redshift/cluster_parameter_group'
    autoload :ClusterSecurityGroup, 'cloud_former/resources/redshift/cluster_security_group'
    autoload :ClusterSecurityGroupIngress, 'cloud_former/resources/redshift/cluster_security_group_ingress'
    autoload :ClusterSubnetGroup, 'cloud_former/resources/redshift/cluster_subnet_group'
    autoload :Parameter, 'cloud_former/resource_properties/redshift/parameter'
  end

  module Route53
    autoload :AliasTarget, 'cloud_former/resource_properties/route53/alias_target'
    autoload :GeoLocation, 'cloud_former/resource_properties/route53/geo_location'
    autoload :HostedZone, 'cloud_former/resources/route53/hosted_zone'
    autoload :HostedZoneConfig, 'cloud_former/resource_properties/route53/hosted_zone_config'
    autoload :HostedZoneTag, 'cloud_former/resource_properties/route53/hosted_zone_tag'
    autoload :HostedZoneVPC, 'cloud_former/resource_properties/route53/hosted_zone_vpc'
    autoload :RecordSet, 'cloud_former/resources/route53/record_set'
    autoload :RecordSetGroup, 'cloud_former/resources/route53/record_set_group'
  end

  module S3
    autoload :Bucket, 'cloud_former/resources/s3/bucket'
    autoload :CorsConfiguration, 'cloud_former/resource_properties/s3/cors_configuration'
    autoload :CorsRule, 'cloud_former/resource_properties/s3/cors_rule'
    autoload :LambdaNotificationConfiguration, 'cloud_former/resource_properties/s3/lambda_notification_configuration'
    autoload :LifecycleConfiguration, 'cloud_former/resource_properties/s3/lifecycle_configuration'
    autoload :LifecycleRule, 'cloud_former/resource_properties/s3/lifecycle_rule'
    autoload :LifecycleRuleTransition, 'cloud_former/resource_properties/s3/lifecycle_rule_transition'
    autoload :LoggingConfiguration, 'cloud_former/resource_properties/s3/logging_configuration'
    autoload :NotificationConfiguration, 'cloud_former/resource_properties/s3/notification_configuration'
    autoload :QueueNotificationConfiguration, 'cloud_former/resource_properties/s3/queue_notification_configuration'
    autoload :TopicNotificationConfiguration, 'cloud_former/resource_properties/s3/topic_notification_configuration'
    autoload :VersioningConfiguration, 'cloud_former/resource_properties/s3/versioning_configuration'
    autoload :WebsiteConfiguration, 'cloud_former/resource_properties/s3/website_configuration'
    autoload(
      :WebsiteConfigurationRedirectAllRequestsTo,
      'cloud_former/resource_properties/s3/website_configuration_redirect_all_requests_to',
    )
    autoload :WebsiteConfigurationRoutingRules, 'cloud_former/resource_properties/s3/website_configuration_routing_rules'
    autoload(
      :WebsiteConfigurationRoutingRulesRedirectRule,
      'cloud_former/resource_properties/s3/website_configuration_routing_rules_redirect_rule',
    )
    autoload(
      :WebsiteConfigurationRoutingRulesRoutingRuleCondition,
      'cloud_former/resource_properties/s3/website_configuration_routing_rules_routing_rule_condition',
    )
  end

  module SNS
    autoload :Subscription, 'cloud_former/resource_properties/sns/subscription'
    autoload :Topic, 'cloud_former/resources/sns/topic'
  end

  module SQS
    autoload :RedrivePolicy, 'cloud_former/resource_properties/sqs/redrive_policy'
    autoload :Queue, 'cloud_former/resources/sqs/queue'
    autoload :QueuePolicy, 'cloud_former/resources/sqs/queue_policy'
  end

  autoload :AllAvailabilityZones, 'cloud_former/all_availability_zones'
  autoload :PropertyOrAttribute, 'cloud_former/property_or_attribute'
  autoload :Template, 'cloud_former/template'
end
