module CloudFormer
  class Resource

    include HasPropertiesAndAttributes
    include MakesJson

    json_attribute :aws_type, name: 'Type'

    attr_reader :aws_type
    attr_reader :metadata_items

    aws_attribute :depends_on, list: true, type: String
    aws_attribute :deletion_policy, type: String
    aws_attribute :condition, type: String

    def initialize(name = nil, &block)
      @name = name
      @metadata_items = []
      @aws_type = self.class.name.sub('CloudFormer', 'AWS')
      if block_given?
        self.instance_eval(&block)
      end
      if @name.nil? || @name.empty?
        raise ArgumentError, "Attempting to create a resource of type #{self.class.name} with no name"
      end
    end

    def get_name
      @name
    end

    def add_metadata(metadata)
      @metadata_items << metadata
    end
  end
end
